﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Capstone.Project
{
    public partial class SentenceCreation : Form
    {
        //set the database string and declare the connection
        private string theConnectionString = "Provider=Microsoft.ACE.OLEDB.12.0;" + @"data source=soundClips.accdb";   // Access 2010
        private OleDbConnection theConnection;

        private const string createdSentence = "createdSentence.wav";

        public SentenceCreation()
        {
            InitializeComponent();            
        }
        
        /******************************
         * When a snippet is double clicked add to the builder list box
         ******************************/ 
        private void listBoxSnippets_DoubleClick(object sender, EventArgs e)
        {
            if (listBoxSnippets.SelectedIndex >= 0)
            {
                listBoxBuilder.Items.Add(listBoxSnippets.SelectedItem.ToString());
            }
        }

        /******************************
         * Build and play the selected snippets
         ******************************/ 
        private void buttonBuildAndPlay_Click(object sender, EventArgs e)
        {
            buttonBuildAndPlay.Enabled = false;
            Queue<string> words = new Queue<string>();
                        
            theConnection = new OleDbConnection(theConnectionString);
            theConnection.Open();

            OleDbCommand searchCommand;
            searchCommand = theConnection.CreateCommand();

            Queue<dataBaseRow> queryResult = new Queue<dataBaseRow>();

            //collect the information for each snippet
            for (int x = 0; x < listBoxBuilder.Items.Count; x++)
            {
                string word = listBoxBuilder.Items[x].ToString();
                string selectStatement = "SELECT * FROM SoundClips WHERE hash = " + word.GetHashCode() + " ";

                searchCommand.CommandText = selectStatement;

                OleDbDataReader databaseReader = searchCommand.ExecuteReader();
                dataBaseRow newRow;

                if (databaseReader.HasRows == true)
                {
                    while (databaseReader.Read())
                    {
                        newRow = new dataBaseRow();

                        newRow.hash = databaseReader.GetInt32(0);
                        newRow.word = databaseReader.GetString(1);
                        newRow.fileLocation = databaseReader.GetString(2);
                        newRow.person = databaseReader.GetString(3);

                        //make sure that the person of the word matches
                        if (newRow.person == comboBoxPerson.SelectedText)
                        {
                            queryResult.Enqueue(newRow);
                        }
                    }
                }
                else
                {
                    MessageBox.Show(word + " is not stored in the database.");
                }

                //need to close the database connection
                databaseReader.Close();
            }

            string outputFile = createdSentence;

            //create the file
            CustomSoundClass.ConcatenateWavFiles(outputFile, queryResult);

            //play the file
            System.Media.SoundPlayer player = new System.Media.SoundPlayer(outputFile);
            player.PlaySync();

            player.Dispose();

            File.Delete(outputFile);

            buttonBuildAndPlay.Enabled = true;
        }

        /******************************
         * The form load event
         * Handles initializing the person comboBox
         ******************************/ 
        private void SentenceCreation_Load(object sender, EventArgs e)
        {
            theConnection = new OleDbConnection(theConnectionString);
            theConnection.Open();

            OleDbCommand searchCommand;
            searchCommand = theConnection.CreateCommand();

            string selectStatement = "SELECT person FROM SoundClips GROUP BY person";

            searchCommand.CommandText = selectStatement;

            OleDbDataReader databaseReader = searchCommand.ExecuteReader();

            if (databaseReader.HasRows == true)
            {

                while (databaseReader.Read())
                {
                    comboBoxPerson.Items.Add(databaseReader.GetString(0));
                }
            }
        }

        /*
         * Handles the selected index change event
         */ 
        private void comboBoxPerson_SelectedIndexChanged(object sender, EventArgs e)
        {
            refreshSnippets();
            listBoxBuilder.Items.Clear();
        }

        /*
         * Creates a new snippet from the currently selected snippets
         */ 
        private void buttonCreateNewSnippet_Click(object sender, EventArgs e)
        {
            //open the database connection
            theConnectionString = "Provider=Microsoft.ACE.OLEDB.12.0;"
                        + @"data source=soundClips.accdb";   // Access 2010
            theConnection = new OleDbConnection(theConnectionString);
            theConnection.Open();

            //get the current person selected
            string personName = comboBoxPerson.Text;

            //assign the tag to nothing for now
            string tag = "";

            //create a queue of databaserows for the snippet concantentate
            Queue<dataBaseRow> selectedSnippets = new Queue<dataBaseRow>();

            //declare the search command
            OleDbCommand searchCommand;

            //set the connection
            searchCommand = theConnection.CreateCommand();
            
            //declare the data reader
            OleDbDataReader reader;

            //run through each snippet that was added to the builder
            for (int x = 0; x < listBoxBuilder.Items.Count; x++)
            {
                //declare a temp databaseRow
                dataBaseRow tempRow = new dataBaseRow();

                //keep building the new tag
                tag += " " + listBoxBuilder.Items[x].ToString();
                
                //run a query to get the information on the current snippet
                searchCommand.CommandText = String.Format("SELECT * FROM SoundClips WHERE hash = {0} AND word = '{1}' AND person = '{2}'", listBoxBuilder.Items[x].ToString().GetHashCode(), listBoxBuilder.Items[x].ToString(), personName);
                reader = searchCommand.ExecuteReader();

                reader.Read();
                //get the hash
                tempRow.hash = reader.GetInt32(0);
                //get the snippet tag
                tempRow.word = reader.GetString(1);
                //get the fileLocation
                tempRow.fileLocation = reader.GetString(2);
                //get the person
                tempRow.person = reader.GetString(3);

                selectedSnippets.Enqueue(tempRow);

                reader.Close();
            }

            tag = tag.TrimStart(' ');

            //make sure that the snippet being created is not already in the database
            searchCommand.CommandText = String.Format("SELECT * FROM SoundClips WHERE hash = {0} AND word = '{1}' AND person = '{2}'", tag.GetHashCode(), tag, personName);
            reader = searchCommand.ExecuteReader();            
            
            if (reader.HasRows == false)
            {
                Console.WriteLine("Word is not in the database.");

                //build the bew filepath
                string filePath = "SoundClipCollection\\" + personName + "\\" + tag + ".wav";

                OleDbCommand insertCommand;
                insertCommand = theConnection.CreateCommand();
                //ID(auto gen), hash-int, word-string, fileLocation-str, person-string
                insertCommand.CommandText = String.Format("INSERT INTO SoundClips VALUES ({0},'{1}','{2}', '{3}')", tag.GetHashCode(), tag, filePath, personName);
                insertCommand.ExecuteNonQuery();                 
                                
                Int32 index = filePath.IndexOf(filePath);

                //make sure that the directory exists
                Directory.CreateDirectory("SoundClipCollection\\" + personName + "\\");

                //concatenate the wav files
                CustomSoundClass.ConcatenateWavFiles(filePath, selectedSnippets);                    

                MessageBox.Show("snippet successfully created");
            }
            else
            {
                MessageBox.Show("Word has already been added to the database.");
                //nothing to delete here since to temp file was created
            }

            //close the connection
            theConnection.Close();

            refreshSnippets();
        }

        /*
         * Refreshes the snippet listbox
         */ 
        private void refreshSnippets()
        {
            listBoxSnippets.Items.Clear();
            theConnection = new OleDbConnection(theConnectionString);
            theConnection.Open();

            OleDbCommand searchCommand;
            searchCommand = theConnection.CreateCommand();

            string selectStatement = "SELECT * FROM SoundClips where person = '" + comboBoxPerson.Text +"'";

            searchCommand.CommandText = selectStatement;

            OleDbDataReader databaseReader = searchCommand.ExecuteReader();

            if (databaseReader.HasRows == true)
            {
                while (databaseReader.Read())
                {
                    string word = databaseReader.GetString(1);
                    string person = databaseReader.GetString(3);

                    //just double checking to be sure that the person is correct
                    if (person == comboBoxPerson.Text)
                    {
                        listBoxSnippets.Items.Add(word);
                    }
                }
            }
            else
            {
                listBoxSnippets.Items.Add("No snippets in the database");
            }
        }

        /*
         * Sends the currently selected snippets to the snippetCreation form
         */ 
        private void buttonEditSnippet_Click(object sender, EventArgs e)
        {
            //need to build the slected snippets into a new file to be opened
            //open the database connection
            theConnectionString = "Provider=Microsoft.ACE.OLEDB.12.0;"
                        + @"data source=soundClips.accdb";   // Access 2010
            theConnection = new OleDbConnection(theConnectionString);
            theConnection.Open();

            //get the current person selected
            string personName = comboBoxPerson.Text;

            //assign the tag to nothing for now
            string tag = "";

            //create a queue of databaserows for the snippet concantentate
            Queue<dataBaseRow> selectedSnippets = new Queue<dataBaseRow>();

            //declare the search command
            OleDbCommand searchCommand;

            //set the connection
            searchCommand = theConnection.CreateCommand();

            //declare the data reader
            OleDbDataReader reader;

            //run through each snippet that was added to the builder
            for (int x = 0; x < listBoxBuilder.Items.Count; x++)
            {
                //declare a temp databaseRow
                dataBaseRow tempRow = new dataBaseRow();


                //run a query to get the information on the current snippet
                searchCommand.CommandText = String.Format("SELECT * FROM SoundClips WHERE hash = {0} AND word = '{1}' AND person = '{2}'", listBoxBuilder.Items[x].ToString().GetHashCode(), listBoxBuilder.Items[x].ToString(), personName);
                reader = searchCommand.ExecuteReader();

                reader.Read();
                //get the hash
                tempRow.hash = reader.GetInt32(0);
                //get the snippet tag
                tempRow.word = reader.GetString(1);
                //get the fileLocation
                tempRow.fileLocation = reader.GetString(2);
                //get the person
                tempRow.person = reader.GetString(3);

                selectedSnippets.Enqueue(tempRow);

                reader.Close();
            }


            string filePath = "fileToLoad.wav";

            //concatenate the wav files
            CustomSoundClass.ConcatenateWavFiles(filePath, selectedSnippets);

            MessageBox.Show("snippet successfully created");            

            snippetCreatorForm child = new snippetCreatorForm(filePath);
            child.ShowDialog();
        }

        //Add to the builder list box
        private void buttonAddSnippet_Click(object sender, EventArgs e)
        {
            if (listBoxSnippets.SelectedIndex >= 0)
            {
                listBoxBuilder.Items.Add(listBoxSnippets.SelectedItem.ToString());
            }
        }

        //remove from the builder box
        private void buttonRemoveSnippet_Click(object sender, EventArgs e)
        {
            if (listBoxBuilder.SelectedIndex >= 0)
            {
                listBoxBuilder.Items.RemoveAt(listBoxBuilder.SelectedIndex);
            }
        }

        //move the snippet up in the builder box
        private void buttonMoveSnippetUp_Click(object sender, EventArgs e)
        {
            if (listBoxBuilder.SelectedIndex >= 1)
            {
                int newIndex = listBoxBuilder.SelectedIndex - 1;

                string newWord = listBoxBuilder.SelectedItem.ToString();

                listBoxBuilder.Items.RemoveAt(listBoxBuilder.SelectedIndex);
                listBoxBuilder.Items.Insert(newIndex, newWord);
            }
        }

        //move the snippet down in the builder box
        private void buttonmoveSnippetDown_Click(object sender, EventArgs e)
        {
            if (listBoxBuilder.SelectedIndex < listBoxBuilder.Items.Count - 1)
            {
                int newIndex = listBoxBuilder.SelectedIndex + 1;

                string newWord = listBoxBuilder.SelectedItem.ToString();

                listBoxBuilder.Items.RemoveAt(listBoxBuilder.SelectedIndex);
                listBoxBuilder.Items.Insert(newIndex, newWord);
            }
        }
    }
}
